#include "drop_area.h"
#include "SDL_events.h"
#include "colour.h"
#include "image.h"
#include "window.h"
#include <stdbool.h>
#include <stdio.h>

void handle_hover(const window_t *wnd, void *obj, const SDL_Event *ev);
void handle_drop_file(const window_t *wnd, void *obj, const SDL_Event *ev);
void render_drop_area(const window_t *wnd, void *obj);
render_rect_t image_render_rect(drop_area_t *area);

void init_drop_area(window_t *wnd, drop_area_t *area, render_rect_t rect,
                    colour_t border_colour) {
  area->rect = rect;
  area->image_rect = (render_rect_t){0};
  area->border_colour = border_colour;
  area->mouseover = false;
  area->image_loaded = false;

  add_event_listener(wnd, area, handle_hover);
  add_event_listener(wnd, area, handle_drop_file);
  add_renderer_func(wnd, area, render_drop_area);
}

void handle_hover(const window_t *wnd, void *obj, const SDL_Event *ev) {
  if (!obj || ev->type != SDL_MOUSEMOTION) {
    return;
  }

  drop_area_t *area = (drop_area_t *)obj;

  area->mouseover = aabb(ev->motion.x, ev->motion.y, &(area->rect));
}

void handle_drop_file(const window_t *wnd, void *obj, const SDL_Event *ev) {
  if (!obj || ev->type != SDL_DROPFILE) {
    return;
  }

  drop_area_t *area = (drop_area_t *)obj;

  if (!(area->mouseover)) {
    return;
  }

  printf("Loading: %s\n", ev->drop.file);

  area->image_loaded = load_image(wnd, &(area->image), ev->drop.file);

  if (area->image_loaded) {
    printf("Width: %ld, Height: %ld\n", area->image.width, area->image.height);
    area->image_rect = image_render_rect(area);
  }
}

void render_drop_area(const window_t *wnd, void *obj) {
  if (!obj) {
    return;
  }

  drop_area_t *area = (drop_area_t *)obj;

  if (area->image_loaded) {
    render_texture(wnd, area->image.texture, NULL, &(area->image_rect));
  }

  draw_rect(wnd, &(area->rect), area->border_colour);
}

render_rect_t image_render_rect(drop_area_t *area) {
  if (!area || !(area->image_loaded)) {
    return (render_rect_t){0};
  }

  u64 w = 0;
  u64 h = 0;

  if (area->image.width < area->rect.width &&
      area->image.height < area->rect.height) {
    w = area->image.width;
    h = area->image.height;
  } else {
    f32 w_factor = (f32)area->rect.width / area->image.width;
    f32 h_factor = (f32)area->rect.height / area->image.height;

    f32 factor = w_factor < h_factor ? w_factor : h_factor;

    w = area->image.width * factor;
    h = area->image.height * factor;
  }

  u64 x = area->rect.x + (area->rect.width - w) / 2;
  u64 y = area->rect.y + (area->rect.height - h) / 2;

  return (render_rect_t){x, y, w, h};
}
